<?php

/**
 * Class ilQuestionSetPoolNodeGUI
 *
 * Date: 21.11.13
 * Time: 14:15
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolNodeGUI extends ilPluginControllerFormGUI
{
    protected $redirectCmd = "ilQuestionSetPoolNodesGUI.listNodes";

    public function edit()
    {
        $model = $this->loadModel();

        $this->redirectIfUnableToAddNewNode($model);
        $this->initForm($model);
        $this->bindModelToForm($model);
        $this->tpl->setContent($this->form->getHTML());
    }

    public function updateAndNew()
    {
        $this->redirectCmd = "ilQuestionSetPoolNodeGUI.edit";
        $this->update();
    }

    public function removeConfirmation()
    {
        $model = $this->loadModel();
        $model->read();

        if (!$model->getId() || $model->getPath()->getSet()->getContainerObjId() != $this->controller->object->getId()) {
            if ($model->getId()) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('nodes_del_perm_err_s'), implode(', ', array($model->getIndex()))), true);
            }
            $this->ctrl->setParameter($this->controller, 'set_id', (int) $_GET['set_id']);
            $this->ctrl->setParameter($this->controller, 'path_id', (int) $_GET['path_id']);
            $this->ctrl->redirect($this->controller, 'ilQuestionSetPoolNodesGUI.listNodes');
            return;
        }

        $confirmation = new ilConfirmationGUI();
        $this->ctrl->setParameter($this->controller, "node_id", $model->getId());
        $this->ctrl->setParameter($this->controller, "set_id", $model->getPath()->getSet()->getId());
        $this->ctrl->setParameter($this->controller, "path_id", $model->getPath()->getId());
        $confirmation->setFormAction($this->ctrl->getFormAction($this->controller, "ilQuestionSetPoolNodesGUI.listNodes"));
        $this->ctrl->setParameter($this->controller, "node_id", "");
        $this->ctrl->setParameter($this->controller, "set_id", "");
        $this->ctrl->setParameter($this->controller, "path_id", "");
        $confirmation->setHeaderText($this->plugin->txt("confirm_delete_node"));
        $confirmation->setCancel($this->plugin->txt("cancel"), "ilQuestionSetPoolNodesGUI.listNodes");
        $confirmation->setConfirm($this->plugin->txt("confirm"), "ilQuestionSetPoolNodeGUI.remove");

        $confirmation->addItem("id", $model->getId(), "(Q" . $model->getQuestionIndex() . ") " . $model->getQuestion()->getTitle());
        $this->tpl->setContent($confirmation->getHTML());
    }

    public function remove()
    {
        $model = $this->loadModel();
        $model->read();

        if (!$model->getId() || $model->getPath()->getSet()->getContainerObjId() != $this->controller->object->getId()) {
            if ($model->getId()) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('nodes_del_perm_err_s'), implode(', ', array($model->getIndex()))), true);
            }
            $this->ctrl->setParameter($this->controller, 'set_id', (int) $_GET['set_id']);
            $this->ctrl->setParameter($this->controller, 'path_id', (int) $_GET['path_id']);
            $this->ctrl->redirect($this->controller, 'ilQuestionSetPoolNodesGUI.listNodes');
            return;
        }

        $data = $model->getPath()->getNodes()->getData();
        foreach ($data as $key => $element) {
            if ($element["id"] == $model->getId()) {
                unset($data[$key]);
            }
        }

        if ($model->getIndex() == 1) {
            ilUtil::sendFailure($this->plugin->txt("cant_delete_first_node"), true);

            $this->redirect(
                "ilQuestionSetPoolNodesGUI.listNodes",
                $this->getAfterUpdateRedirectParams($model)
            );
        }

        $canBeDeleted = true;
        foreach ($data as $node) {
            $node = new ilQuestionSetPoolNode($node["id"]);
            $node->read();

            if (!ilQuestionSetPoolConditionValidator::isValid($node->getInitialCondition(), $node, $data, $this->plugin)) {
                $canBeDeleted = false;
                ilUtil::sendFailure($this->plugin->txt("node_used_in_initial_condition"), true);
            }

            $conditions = $node->getJumpConditionList()->getData();
            foreach ($conditions as $condition) {
                if (!ilQuestionSetPoolConditionValidator::isValid($condition["jump_condition"], $node, $data, $this->plugin)) {
                    $canBeDeleted = false;
                    ilUtil::sendFailure($this->plugin->txt("node_used_in_jump_condition"), true);
                }

                if ($condition["question_fi"] == $model->getQuestion()->getId() &&
                    $condition["node_fi"] != $model->getId()
                ) {
                    $canBeDeleted = false;
                    ilUtil::sendFailure($this->plugin->txt("node_is_used_as_target"), true);
                }
            }
        }

        if ($canBeDeleted) {
            $model->remove();
            ilUtil::sendSuccess($this->plugin->txt("node_deleted"), true);
        }

        $this->redirect(
            "ilQuestionSetPoolNodesGUI.listNodes",
            $this->getAfterUpdateRedirectParams($model)
        );
    }

    /**
     * Loads and returns the Model for the controller gui
     * @return ilQuestionSetPoolNode
     */
    protected function loadModel()
    {
        $id = (isset($_GET['node_id']))? (int) $_GET['node_id'] : (int) $_POST['id'];
        $model = new ilQuestionSetPoolNode($id);
        $model->read();

        return $model;
    }

    /**
     * Initialize the PropertyFormGUI for this controller
     *
     * @param $model
     */
    protected function initForm($model)
    {
        $this->form = new ilPropertyFormGUI();
        $this->form->setTitle(sprintf(
            $this->plugin->txt("set_node_title"),
            $model->getPath()->getSet()->getTitle(),
            $model->getPath()->getTitle()
            ));

        $question = new ilSelectInputGUI($this->plugin->txt('question'), "question_fi");
        $question->setRequired(true);
        $question->setOptions($this->getQuestionOptions($model));

        $this->form->addItem($question);

        $this->ctrl->setParameter($this->controller, "path_id", (int) $_GET['path_id']);
        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET['set_id']);
        $this->form->setFormAction($this->ctrl->getFormAction($this->controller));
        $this->ctrl->setParameter($this->controller, "path_id", "");
        $this->ctrl->setParameter($this->controller, "set_id", "");
        $this->form->addCommandButton("ilQuestionSetPoolNodeGUI.updateAndNew", $this->plugin->txt("save_and_new"));
        $this->form->addCommandButton("ilQuestionSetPoolNodeGUI.update", $this->plugin->txt("save"));
        $this->form->addCommandButton("ilQuestionSetPoolNodesGUI.listNodes", $this->plugin->txt("cancel"));
    }

    /**
     * Get the redirect action after an update was successful
     * @return string
     */
    protected function getAfterUpdateRedirectCmd()
    {
        ilUtil::sendSuccess($this->plugin->txt("msg_node_created"), true);
        return $this->redirectCmd;
    }

    /**
     * @param $model
     *
     * @return array
     */
    protected function getAfterUpdateRedirectParams($model)
    {
        return array(
            "path_id" => $model->getPath()->getId(),
            "set_id" => $model->getPath()->getSet()->getId()
        );
    }

    /**
     * Returns the current tab name
     * return string
     */
    protected function getCurrentTab()
    {
        return "content";
    }

    /**
     * @return array
     */
    protected function getRequirements()
    {
        return array(
            "models/class.ilQuestionSetPoolNode.php",
            "models/class.ilQuestionSetPoolPath.php",
            "utils/abstract.ilQuestionSetPoolConditionValidator.php"
        );
    }

    /**
     * @param ilQuestionSetPoolNode $model
     *
     * @return array
     */
    protected function getQuestionOptions($model)
    {
        $path = new ilQuestionSetPoolPath((int) $_GET['path_id']);
        $path->read();
        $nodes = $path->getNodes()->getData();
        $questions = $model->getPath()->getSet()->getQuestionList()->getData();
        $options = array();

        foreach ($questions as $question) {
            $options[$question['question_id']] = $question['title'];
        }

        foreach ($nodes as $node) {
            if (array_key_exists($node["question_fi"], $options)) {
                unset($options[$node["question_fi"]]);
            }
        }

        return $options;
    }

    /**
     * @param $model
     */
    protected function redirectIfUnableToAddNewNode($model)
    {
        if (is_array($this->getQuestionOptions($model)) && count($this->getQuestionOptions($model)) == 0) {
            ilUtil::sendFailure($this->plugin->txt("unable_to_add_new_node"), true);
            $this->redirect(
                "ilQuestionSetPoolNodesGUI.listNodes",
                array(
                         "path_id" => (int) $_GET["path_id"],
                         "set_id" => (int) $_GET["set_id"]
                     )
            );
        }
    }
}
