<?php

/**
 * Class ilQuestionSetPoolPathsGUI
 *
 * Date: 08.10.13
 * Time: 11:11
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolPathsGUI extends ilPluginControllerTableGUI
{
    public function __construct(ilObjQuestionSetPoolGUI $controller)
    {
        parent::__construct($controller);
        $this->addSubTabs();
    }

    public function listPaths()
    {
        if ($this->controller->object instanceof ilObjQuestionSetPool) {
            $this->addToolbarButton("back_to_set_list", "ilQuestionSetPoolSetsGUI.show");
        }

        $this->addToolbarButton("add_path", "ilQuestionSetPoolPathGUI.edit", array(
            "set_id" => (int) $_GET['set_id']
        ));

        $this->ctrl->setParameter($this->controller, 'set_id', (int) $_GET['set_id']);
        $table = new ilQuestionSetPoolPathTableGUI($this->controller, $this->plugin, (int) $_GET['set_id']);

        $set = new ilQuestionSetPoolSet((int) $_GET['set_id']);
        $set->read();

        $table->setModelList($set->getPathList());
        $table->setTitle($set->getTitle() . " - " . $this->plugin->txt('paths'));
        $table->addCommandButton("ilQuestionSetPoolPathsGUI.saveSorting", $this->plugin->txt("save_sorting"));
        $table->populate();

        $this->tpl->setContent($table->getHTML());
    }

    public function saveSorting()
    {
        $this->doSorting();

        ilUtil::sendSuccess($this->plugin->txt("saved_sorting"), true);
        $this->redirect("ilQuestionSetPoolPathsGUI.listPaths", array(
            "set_id" => (int) $_GET["set_id"]
        ));
    }

    public function copy()
    {
        if ($this->dataSelected("path_id")) {
            $set = new ilQuestionSetPoolSet((int) $_GET['set_id']);
            $set->read();
            $path_titles = $set->getPathTitles();
            foreach ($_POST["path_id"] as $path_id) {
                $path = new ilQuestionSetPoolPath($path_id);
                $path->read();

                $data = $path->getSet()->getQuestionList()->getData();
                $question_ids = array();
                foreach ($data as $element) {
                    $question_ids[$element["question_id"]] = $element["question_id"];
                }

                $title = $path->getTitle();
                $i = 2;
                while (in_array($title . ' (' . $i . ')', $path_titles)) {
                    $i++;
                }

                $title .= ' (' . $i . ')';

                $path_titles[] = $title;

                $path->duplicate($question_ids, $title);
            }

            //$this->doSorting();

            ilUtil::sendInfo($this->plugin->txt('success_path_copy'));
            $this->redirect("ilQuestionSetPoolPathsGUI.listPaths", array(
                "set_id" => $_GET["set_id"]
            ));
        } else {
            ilUtil::sendInfo($this->plugin->txt('no_path_selected'));
            $this->listPaths();
            return;
        };
    }

    /**
     * Called if a single path should be deleted, calls \ilQuestionSetPoolPathsGUI::removeMultipleConfirmation do show the confirmation screen
     */
    public function removeConfirmation()
    {
        if (!isset($_GET['path_id'])) {
            ilUtil::sendInfo($this->plugin->txt('no_path_selected'));
            $this->listPaths();
            return;
        }

        $_POST['path_id'] = array((int) $_GET['path_id']);
        $this->removeMultipleConfirmation();
    }

    /**
     * Shows an confirmation screen for the selected paths
     */
    public function removeMultipleConfirmation()
    {
        if (!isset($_POST['path_id']) || !is_array($_POST['path_id'])) {
            ilUtil::sendInfo($this->plugin->txt('no_path_selected'));
            $this->listPaths();
            return;
        }

        $this->tabs->activateSubTab('paths');

        $set = new ilQuestionSetPoolSet((int) $_GET["set_id"]);
        $set->read();

        $confirmation = new ilConfirmationGUI();

        $num_items = 0;
        $missing_perm_path_titles = array();
        foreach ($_POST['path_id'] as $path_id) {
            $path = new ilQuestionSetPoolPath((int) $path_id);
            $path->read();
            if ($path->getSet()->getContainerObjId() == $this->controller->object->getId()) {
                $confirmation->addItem('path_id[]', $path->getId(), $path->getTitle());
                ++$num_items;
            } else {
                $missing_perm_path_titles[] = $path->getTitle();
            }
        }

        if (1 == $num_items) {
            $confirmation->setHeaderText($this->plugin->txt('confirm_delete_paths_s'));
        } elseif ($num_items > 1) {
            $confirmation->setHeaderText($this->plugin->txt('confirm_delete_paths_p'));
        }

        $missing_perm_path_titles = array_filter($missing_perm_path_titles);
        if (is_array($missing_perm_path_titles)) {
            if (1 == count($missing_perm_path_titles)) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('paths_del_perm_err_s'), implode(', ', $missing_perm_path_titles)));
            } elseif (count($missing_perm_path_titles) > 1) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('paths_del_perm_err_p'), implode(', ', $missing_perm_path_titles)));
            }
        }
        if (!$num_items) {
            $this->listPaths();
            return;
        }

        $this->ctrl->setParameter($this->controller, 'set_id', $set->getId());
        $confirmation->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.listPaths'));
        $this->ctrl->setParameter($this->controller, 'set_id', '');
        $confirmation->setCancel($this->plugin->txt('cancel'), __CLASS__ . '.listPaths');
        $confirmation->setConfirm($this->plugin->txt('confirm'), __CLASS__ . '.remove');
        $this->tpl->setContent($confirmation->getHTML());
    }

    /**
     * Finally deletes (cascading) the selected paths
     */
    public function remove()
    {
        if (!isset($_POST['path_id']) || !is_array($_POST['path_id'])) {
            ilUtil::sendInfo($this->plugin->txt('no_path_selected'));
            $this->listPaths();
            return;
        }

        $set = new ilQuestionSetPoolSet((int) $_GET["set_id"]);
        $set->read();

        $num_items = 0;
        $missing_perm_path_titles = array();
        foreach ($_POST['path_id'] as $path_id) {
            $path = new ilQuestionSetPoolPath((int) $path_id);
            $path->read();
            if ($path->getSet()->getContainerObjId() == $this->controller->object->getId()) {
                ++$num_items;
                $path->remove();
            } else {
                $missing_perm_path_titles[] = $path->getTitle();
            }
        }

        if (1 == $num_items) {
            ilUtil::sendSuccess($this->plugin->txt('deleted_paths_s'), true);
        } elseif ($num_items > 1) {
            ilUtil::sendSuccess($this->plugin->txt('deleted_paths_p'), true);
        }

        $missing_perm_path_titles = array_filter($missing_perm_path_titles);
        if (is_array($missing_perm_path_titles)) {
            if (1 == count($missing_perm_path_titles)) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('paths_del_perm_err_s'), implode(', ', $missing_perm_path_titles)));
            } elseif (count($missing_perm_path_titles) > 1) {
                ilUtil::sendFailure(sprintf($this->plugin->txt('paths_del_perm_err_p'), implode(', ', $missing_perm_path_titles)));
            }
        }
        $this->ctrl->setParameter($this->controller, 'set_id', $set->getId());
        $this->redirect(__CLASS__ . '.listPaths');
    }

    /**
     * Returns the current tab name
     * return string
     */
    protected function getCurrentTab()
    {
        return "content";
    }

    /**
     * @return array
     */
    protected function getRequirements()
    {
        return array(
            "tables/class.ilQuestionSetPoolPathTableGUI.php",
            "models/class.ilQuestionSetPoolPath.php",
            "models/class.ilQuestionSetPoolSet.php"
        );
    }

    private function addSubTabs()
    {
        global $DIC;
        $ilTabs = $DIC->tabs();

        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET['set_id']);

        $ilTabs->addSubTab(
            "set_questions",
            $this->plugin->txt("questions"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolSetQuestionsGUI.listQuestions")
        );

        $ilTabs->addSubTab(
            "paths",
            $this->plugin->txt("paths"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.listPaths")
        );

        $ilTabs->addSubTab(
            "path_overview",
            $this->plugin->txt("path_overview"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGraphGUI.showPathOverview")
        );

        $ilTabs->activateSubTab("paths");

        $this->ctrl->setParameter($this->controller, "set_id", "");
    }

    protected function doSorting()
    {
        $model = new ilQuestionSetPoolSet((int) $_GET['set_id']);
        $model->read();
        $data = $model->getPathList()->getData();

        foreach ($data as $key => $element) {
            $data[$key]["path_index"] = (int) $_POST["sorting"][$element["id"]];
        }

        usort(
            $data,
            function ($a, $b) {
                return $a["path_index"] - $b["path_index"];
            }
        );
        if (is_array($data)) {
            for ($i = 0; $i < count($data); $i++) {
                $data[$i]["path_index"] = ($i + 1);
                $model->getPathList()->update($data[$i]);
            }
        }
    }
}
