<?php

/**
 * Class ilQuestionSetPoolSetGUI
 *
 * Date: 02.10.13
 * Time: 10:01
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolSetGUI extends ilPluginControllerFormGUI
{
    public function update()
    {
        $model = $this->loadModel();
        $this->initForm($model);
        if ($this->form->checkInput()) {
            $model->bindForm($this->form);

            if ($model->isTitleUnique()) {
                $model->update();
                ilUtil::sendSuccess($this->plugin->txt("msg_obj_modified"), true);
                $this->redirect(
                    $this->getAfterUpdateRedirectCmd(),
                    $this->getAfterUpdateRedirectParams($model)
                );
            } else {
                ilUtil::sendFailure($this->plugin->txt("set_title_not_unique"), true);
            }
        }

        $this->form->setValuesByPost();
        $this->tpl->setContent($this->form->getHTML());
    }

    /**
     * Returns the current tab name
     * return string
     */
    protected function getCurrentTab()
    {
        return "content";
    }

    /**
     * @return array
     */
    protected function getRequirements()
    {
        return array(
            "models/class.ilQuestionSetPoolSet.php"
        );
    }

    protected function initForm($model)
    {
        $this->form = new ilPropertyFormGUI();
        $this->form->setTitle(sprintf(
            $this->plugin->txt("question_set_title"),
            $this->controller->object->getTitle()
        ));

        $id = new ilHiddenInputGUI("id");

        $title = new ilTextInputGUI($this->plugin->txt("title"), "title");
        $title->setRequired(true);

        $description = new ilTextAreaInputGUI($this->plugin->txt("description"), "description");
        $description->setRequired(false);

        $this->form->addItem($id);
        $this->form->addItem($title);
        $this->form->addItem($description);


        $this->form->setFormAction($this->ctrl->getFormAction($this->controller));

        $this->form->addCommandButton("ilQuestionSetPoolSetGUI.update", $this->plugin->txt("save"));
        $this->form->addCommandButton("ilQuestionSetPoolSetsGUI.show", $this->plugin->txt("cancel"));
    }

    /**
     * Loads and returns the Model for the controller gui
     *
     * @return ilPluginModelInterface
     */
    protected function loadModel()
    {
        $id = isset($_GET["set_id"])? (int) $_GET['set_id']: (int) $_POST["id"];
        $model = new ilQuestionSetPoolSet($id);
        $model->setContainerObjId($this->controller->object->getId());
        $model->read();

        return $model;
    }

    /**
     * Get the redirect action after an update was successful
     * @return string
     */
    protected function getAfterUpdateRedirectCmd()
    {
        return "ilQuestionSetPoolSetsGUI.show";
    }

    /**
     * @param $model
     *
     * @return array
     */
    protected function getAfterUpdateRedirectParams($model)
    {
        return array();
    }
}
