<?php

/**
 * Class ilQuestionSetPoolJumpCondition
 *
 * Date: 04.12.13
 * Time: 17:00
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolJumpCondition extends ilPluginModel
{
    /**
     * @var string
     */
    protected $jump_condition;

    /**
     * @var assQuestion
     */
    private $question;

    /**
     * @var ilQuestionSetPoolNode
     */
    protected $node;

    /**
     * @var int
     */
    protected $repititions = null;

    /**
     * @var int
     */
    protected $sorting;

    /**
     * @var string null
     */
    protected $true_feedback = null;

    /**
     * @var string null
     */
    protected $true_jump_feedback = null;

    /**
     * @var string null
     */
    protected $false_feedback = null;

    /**
     * @param string $jump_condition
     */
    public function setJumpCondition($jump_condition)
    {
        $this->jump_condition = $jump_condition;
    }

    /**
     * @return string
     */
    public function getJumpCondition()
    {
        return $this->jump_condition;
    }

    /**
     * @param \ilQuestionSetPoolNode $node
     */
    public function setNode($node)
    {
        $this->node = $node;
    }

    /**
     * @return \ilQuestionSetPoolNode
     */
    public function getNode()
    {
        return $this->node;
    }

    /**
     * @param \assQuestion $question
     */
    public function setQuestion($question)
    {
        $this->question = $question;
    }

    /**
     * @return \assQuestion
     */
    public function getQuestion()
    {
        return $this->question;
    }

    /**
     * @param int $repititions
     */
    public function setRepititions($repititions)
    {
        $this->repititions = $repititions;
    }

    /**
     * @return int
     */
    public function getRepititions()
    {
        return $this->repititions;
    }

    /**
     * @param int $sorting
     */
    public function setSorting($sorting)
    {
        $this->sorting = $sorting;
    }

    /**
     * @return int
     */
    public function getSorting()
    {
        return $this->sorting;
    }
    
    public function update()
    {
        global $DIC;
        $ilDB = $DIC->database();

        $question_id = null;
        if ($this->getQuestion() != null) {
            $question_id = $this->getQuestion()->getId();
        }

        if ($this->id > 0) {
            return (bool) $ilDB->update(
                "rep_robj_xqsp_jmp_con",
                array(
                    "question_fi" => array("integer", $question_id),
                    "jump_condition" => array("text", $this->getJumpCondition()),
                    "repititions" => array("integer", $this->getRepititions()),
                    "sorting" => array("integer", $this->getSorting()),
                    'true_feedback' => array('clob', $this->getTrueFeedback()),
                    'true_jmp_feedback' => array('clob', $this->getTrueJumpFeedback()),
                    'false_feedback' => array('clob', $this->getFalseFeedback()),
                ),
                array(
                    "id" => array("integer", $this->getId())
                )
            );
        } else {
            $this->setId($ilDB->nextId("rep_robj_xqsp_jmp_con"));
            
            $this->setSorting(time());
            return $ilDB->insert(
                "rep_robj_xqsp_jmp_con",
                array(
                    "id" => array("integer", $this->getId()),
                    "jump_condition" => array("text", $this->getJumpCondition()),
                    "node_fi" => array("integer", $this->getNode()->getId()),
                    "question_fi" => array("integer", $question_id),
                    "repititions" => array("integer", $this->getRepititions()),
                    "sorting" => array("integer", $this->getSorting()),
                    'true_feedback' => array('clob', $this->getTrueFeedback()),
                    'true_jmp_feedback' => array('clob', $this->getTrueJumpFeedback()),
                    'false_feedback' => array('clob', $this->getFalseFeedback()),
                )
            );
        }
    }

    public function read()
    {
        global $DIC;
        $ilDB = $DIC->database();

        $result = $ilDB->queryF(
            "SELECT * FROM rep_robj_xqsp_jmp_con WHERE id = %s",
            array("integer"),
            array($this->id)
        );
        $data = $ilDB->fetchAssoc($result);

        $id = (isset($data['node_fi']))? $data['node_fi'] : $_REQUEST['node_fi'];
        $node = new ilQuestionSetPoolNode($id);
        $node->read();

        $question = assQuestion::_instanciateQuestion($data["question_fi"]);

        $this->setId($data['id']);
        $this->setJumpCondition($data['jump_condition']);
        $this->setQuestion($question);
        $this->setNode($node);
        $this->setRepititions($data["repititions"]);
        $this->setSorting($data["sorting"]);

        $this->setTrueFeedback(ilRTE::_replaceMediaObjectImageSrc($data['true_feedback'], 1));
        $this->setTrueJumpFeedback(ilRTE::_replaceMediaObjectImageSrc($data['true_jmp_feedback'], 1));
        $this->setFalseFeedback(ilRTE::_replaceMediaObjectImageSrc($data['false_feedback'], 1));
    }

    /**
     * @param ilQuestionSetPoolNode $node
     * @param array $question_ids
     *
     * @return bool
     */
    public function duplicate($node, $question_ids)
    {
        global $DIC;
        $ilDB = $DIC->database();
        $clone = clone $this;
        $clone->setId($ilDB->nextId("rep_robj_xqsp_jmp_con"));
        $clone->setNode($node);
        $clone->setSorting($this->getSorting());

        $question_id = null;
        if ($this->getQuestion() != null) {
            $question_id = $this->getQuestion()->getId();
        }

        return $ilDB->insert(
            "rep_robj_xqsp_jmp_con",
            array(
                "id" => array("integer", $clone->getId()),
                "jump_condition" => array("text", $clone->getJumpCondition()),
                "question_fi" => array("integer", $question_ids[$question_id]),
                "node_fi" => array("integer", $clone->getNode()->getId()),
                "repititions" => array("integer", $this->getRepititions()),
                "sorting" => array("integer", $this->getSorting()),
                'true_feedback' => array('clob', $this->getTrueFeedback()),
                'true_jmp_feedback' => array('clob', $this->getTrueJumpFeedback()),
                'false_feedback' => array('clob', $this->getFalseFeedback()),
            )
        );
    }

    public function remove()
    {
        global $DIC;
        $ilDB = $DIC->database();

        return $ilDB->manipulateF(
            "DELETE FROM rep_robj_xqsp_jmp_con WHERE id = %s",
            array("integer"),
            array($this->getId())
        );
    }

    /**
     * @param ilPropertyFormGUI $form
     */
    public function bindForm(ilPropertyFormGUI $form)
    {
        $this->setJumpCondition($form->getInput("jump_condition"));
        $this->setQuestion(assQuestion::_instanciateQuestion($form->getInput("question")));
        if ($form->getInput("repititions") != "") {
            $this->setRepititions($form->getInput("repititions"));
        } else {
            $this->setRepititions(null);
        }

        $this->setTrueFeedback(ilRTE::_replaceMediaObjectImageSrc($form->getInput('true_feedback'), 0));
        $this->setTrueJumpFeedback(ilRTE::_replaceMediaObjectImageSrc($form->getInput('true_jump_feedback'), 0));
        $this->setFalseFeedback(ilRTE::_replaceMediaObjectImageSrc($form->getInput('false_feedback'), 0));
    }
    /**
     * @return string
     */
    public function getFalseFeedback()
    {
        return $this->false_feedback;
    }

    /**
     * @param string $false_feedback
     */
    public function setFalseFeedback($false_feedback)
    {
        $this->false_feedback = $false_feedback;
    }

    /**
     * @return string
     */
    public function getTrueFeedback()
    {
        return $this->true_feedback;
    }

    /**
     * @param string $true_feedback
     */
    public function setTrueFeedback($true_feedback)
    {
        $this->true_feedback = $true_feedback;
    }

    /**
     * @return string
     */
    public function getTrueJumpFeedback()
    {
        return $this->true_jump_feedback;
    }

    /**
     * @param string $true_jump_feedback
     */
    public function setTrueJumpFeedback($true_jump_feedback)
    {
        $this->true_jump_feedback = $true_jump_feedback;
    }
    
    public function import()
    {
        global $DIC;
        $ilDB = $DIC->database();
        B;
        
        $this->setId($ilDB->nextId("rep_robj_xqsp_jmp_con"));

        $question_id = null;
        if ($this->getQuestion() != null) {
            $question_id = $this->getQuestion()->getId();
        }

        $this->setSorting(time());
        $ilDB->insert(
            "rep_robj_xqsp_jmp_con",
            array(
                        "id" => array("integer", $this->getId()),
                        "jump_condition" => array("text", $this->getJumpCondition()),
                        "node_fi" => array("integer", $this->getNode()->getId()),
                        "question_fi" => array("integer", $question_id),
                        "repititions" => array("integer", $this->getRepititions()),
                        "sorting" => array("integer", $this->getSorting()),
                        'true_feedback' => array('clob', $this->getTrueFeedback()),
                        'true_jmp_feedback' => array('clob', $this->getTrueJumpFeedback()),
                        'false_feedback' => array('clob', $this->getFalseFeedback())
                )
        );
    }
}
