<?php

/**
 * Class ilQuestionSetPoolPathTableGUI
 *
 * Date: 18.11.13
 * Time: 13:01
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolPathTableGUI extends ilPluginTableGUI
{
    private $controller;

    public function __construct($controller, $plugin, $set_id)
    {
        global $DIC;
    
        $this->ctrl = $DIC->ctrl();
        $this->controller = $controller;
        $this->setId("xqsp_paths_" . $plugin->getId() . '_' . $set_id);

        parent::__construct($controller, "", $plugin);

        $this->createTable();
    }

    private function createTable()
    {
        $this->setTitle($this->plugin->txt('paths'));

        $this->setFormName("paths");
        $this->setStyle('table', 'fullwidth');
        $this->addColumn('', '', '1%', true);
        $this->addColumn($this->plugin->txt('index'), '', true, '1%');
        $this->addColumn($this->lng->txt('title'));
        $this->addColumn($this->lng->txt('actions'), '', '1%');

        $this->setSelectAllCheckbox('path_id');

        $this->setRowTemplate('tpl.il_xqsp_path_row.html', $this->controller->plugin->getDirectory());
        $this->setFormAction($this->ctrl->getFormAction($this->parent_obj));
        $this->setDefaultOrderField("path_index");
        $this->setDefaultOrderDirection("asc");

        $this->addMultiCommand('ilQuestionSetPoolPathsGUI.removeMultipleConfirmation', $this->lng->txt('delete'));
        $this->addMultiCommand('ilQuestionSetPoolPathsGUI.copy', $this->lng->txt('copy'));

        //$this->enable('sort');
        $this->disable("sort");
        $this->enable('header');
        $this->enable('select_all');
    }


    /**
     * Return an array of all static (always visible) data fields in a row.
     * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
     * Example:
     *     return array('title', 'checkbox');
     *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
     * @return array
     */
    protected function getStaticData()
    {
        return array(
            'path_index',
            'checkbox',
            'title_link',
            'title',
            'actions'
        );
    }

    protected function formatCellValue($column, array $row)
    {
        if ($column == "path_index") {
            $sort = new ilNumberInputGUI('', 'sorting[' . $row['id'] . ']');
            $sort->allowDecimals(false);
            $sort->setSize(3);
            $sort->setValue($row[$column]);
            $row[$column] = $sort->render();
        }
        if ($column == "checkbox") {
            $row[$column] = ilUtil::formCheckbox(false, 'path_id[]', $row['id']);
        }
        if ($column == "title_link") {
            $this->ctrl->setParameter($this->controller, "path_id", $row['id']);
            $this->ctrl->setParameter($this->controller, "set_id", $row['questionset_fi']);
            $href = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolNodesGUI.listNodes");
            $this->ctrl->setParameter($this->controller, "path_id", "");
            $this->ctrl->setParameter($this->controller, "set_id", $row['questionset_fi']);
            $row[$column] = $href;
        } elseif ('actions' == $column) {
            $action = new ilAdvancedSelectionListGUI();
            $action->setId('asl_' . $row["id"]);
            $action->setListTitle($this->lng->txt('actions'));

            $this->ctrl->setParameter($this->controller, "path_id", $row['id']);
            $this->ctrl->setParameter($this->controller, "set_id", $row['questionset_fi']);
            $action->addItem($this->lng->txt("edit"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGUI.edit"));
            $action->addItem($this->plugin->txt("show_nodes"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolNodesGUI.listNodes"));
            $action->addItem($this->lng->txt("delete"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.removeConfirmation"));
            $this->ctrl->setParameter($this->controller, "path_id", "");
            $this->ctrl->setParameter($this->controller, "set_id", $row['questionset_fi']);

            $row["actions"] = $action->getHtml();
        }

        return parent::formatCellValue($column, $row);
    }
}
