<?php

/**
 * Class ilQuestionSetPoolQuestionsTable
 *
 * Date: 23.09.13
 * Time: 16:20
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolQuestionsTableGUI extends ilPluginTableGUI
{
    const CONTEXT_BROWSE_QST_TO_LINK = 2;
    const CONTEXT_LIST_LINKED_QST_IN_SET = 3;
    const CONTEXT_LIST_SET_QST = 4;

    /**
     * @var array
     */
    protected static $checked_qst_types = array();

    /**
     * @var ilObjQuestionSetPoolGUI
     */
    private $controller;
    
    /**
     * @var string
     */
    public $gui_class_name = "";

    /**
     * @var int
     */
    protected $table_context = self::CONTEXT_BROWSE_QST_TO_LINK;

    /**
     * @param ilObjQuestionSetPoolGUI $controller
     * @param ilQuestionSetPoolPlugin $plugin
     * @param string $action
     * @param int $context
     * @param int|null $set_id
     */
    public function __construct(ilObjQuestionSetPoolGUI $controller, ilQuestionSetPoolPlugin $plugin, $action, $context = self::CONTEXT_BROWSE_QST_TO_LINK, $set_id = null)
    {
        global $DIC;
        
        $this->ctrl = $DIC->ctrl();
        $this->controller = $controller;
        $this->setTableContext($context);
        $this->setId("xqsp_qst_" . $plugin->getId() . ($set_id ? '_' . $set_id : ''));

        parent::__construct($controller, $action, $plugin);

        $this->createTable();
    }

    /**
     * @param int $context
     */
    public function setTableContext($context)
    {
        $this->table_context = $context;
    }

    /**
     * @return int
     */
    public function getTableContext()
    {
        return $this->table_context;
    }

    /**
     *
     */
    private function createTable()
    {
        $this->setTitle($this->plugin->txt("questionbrowser"));

        $this->setFormName('questionbrowser');
        $this->setStyle('table', 'fullwidth');
        $this->addColumn('', '', '1%', true);

        if ($this->getTableContext() == self::CONTEXT_LIST_SET_QST) {
            $this->optionalColumns = array(
                "question_index" => "question_index"
            );
            $this->visibleOptionalColumns = array(
                "question_index" => true
            );

            $this->addColumn($this->plugin->txt("index"), 'index', '');
            $this->setDefaultOrderField("index");
        } else {
            $this->setDefaultOrderField("title");
        }
        $this->addColumn($this->lng->txt("title"), 'title', '');
        $this->addColumn($this->lng->txt("description"), 'description', '');
        $this->addColumn($this->plugin->txt("question_type"), 'type_tag', '');
        $this->addColumn($this->plugin->txt("points"), 'points', '');
        $this->addColumn($this->lng->txt("author"), 'author', '');
        $this->addColumn($this->lng->txt("create_date"), 'created', '');
        $this->addColumn($this->lng->txt("last_update"), 'tstamp', '');
        $this->addColumn($this->plugin->txt("original_question_pool"), 'qpl', '');
        $this->addColumn($this->lng->txt("actions"), '', '');

        $this->setSelectAllCheckbox('q_id');

        $this->setRowTemplate("tpl.il_xqsp_questions_row.html", $this->controller->plugin->getDirectory());

        $this->setFormAction($this->ctrl->getFormAction($this->controller));
        $this->setDefaultOrderDirection("asc");

        $this->enable('sort');
        $this->enable('header');
        $this->enable('select_all');
    }

    public function setGuiClassName($gui_class_name)
    {
        $this->gui_class_name = $gui_class_name;
    }
    /**
     * @param string $gui_class_name
     */
    public function initFilter()
    {
        $this->createFilterTextInput("title");
        $this->createFilterTextInput("description");
        $this->createFilterTextInput("author");
        $this->createFilterTextInput("qpl");
        $this->createFilterSelectInput("question_type", $this->pullQuestionTypes());

        $this->setFilterCommand($this->gui_class_name . '.applyAvailableFilter');
        $this->setResetCommand($this->gui_class_name . '.resetAvailableFilter');
    }

    /**
     * @param $caption
     */
    private function createFilterTextInput($caption)
    {
        $input = new ilTextInputGUI($this->lng->txt($caption), $caption);
        $input->setMaxLength(64);
        $input->setSize(20);
        $input->setValidationRegexp('/^[^%]+$/is');
        $this->addFilterItem($input);
        $input->readFromSession();
        $this->filter[$caption] = $input->getValue();
    }

    /**
     * @param string $caption
     * @param array  $options
     */
    private function createFilterSelectInput($caption, $options)
    {
        $si = new ilSelectInputGUI($this->lng->txt($caption), $caption);
        $si->setOptions($options);
        $this->addFilterItem($si);
        $si->readFromSession();
        $this->filter[$caption] = $si->getValue();
    }

    /**
     * @return array
     */
    private function pullQuestionTypes()
    {
        $types = ilObjQuestionPool::_getQuestionTypes();
        $options = array(
            "" => $this->lng->txt('filter_all_question_types')
        );
        foreach ($types as $translation => $row) {
            $options[$row['type_tag']] = $translation;
        }

        return $options;
    }

    /**
     * Return an array of all static (always visible) data fields in a row.
     * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
     * Example:
     *     return array('title', 'checkbox');
     *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
     * @return array
     */
    protected function getStaticData()
    {
        return array(
            'title',
            "description",
            "author",
            "type_tag",
            "points",
            "question_id",
            "tstamp",
            "created",
            "pool_title",
            "actions"
        );
    }

    public function numericOrdering($a_field)
    {
        return in_array($a_field, array(
            "points", "tstamp", "created"
        ));
    }


    /**
     * Define a final formatting for a cell value
     *
     * @param mixed $column
     * @param array $row
     *
     * @return mixed
     */
    protected function formatCellValue($column, array $row)
    {
        if ($column == "tstamp" || $column == "created") {
            return ilDatePresentation::formatDate(new ilDateTime($row[$column], IL_CAL_UNIX));
        } elseif ($column == "type_tag") {
            return assQuestion::_getQuestionTypeName($row[$column]);
        } elseif ($column == "actions") {
            $action = new ilAdvancedSelectionListGUI();
            $action->setId('asl_' . $row["question_id"]);
            $action->setListTitle($this->lng->txt('actions'));

            $this->ctrl->setParameter($this->parent_obj, "q_id", $row["question_id"]);
            if (in_array($this->getTableContext(), array(self::CONTEXT_LIST_LINKED_QST_IN_SET, self::CONTEXT_BROWSE_QST_TO_LINK))) {
                $this->ctrl->setParameter($this->parent_obj, "referer", 'browse');
            }

            if ($this->getTableContext() == self::CONTEXT_LIST_LINKED_QST_IN_SET) {
                $action->addItem($this->lng->txt("preview"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetQuestionsGUI.showQuestionPreview"));
            }
            $this->ctrl->setParameter($this->parent_obj, "q_id", "");

            $this->ctrl->setParameterByClass('ilObjQuestionPoolGUI', 'ref_id', $row["pool_ref_id"]);
            $this->ctrl->setParameterByClass('ilObjQuestionPoolGUI', 'q_id', $row["question_id"]);
            $action->addItem(
                $this->plugin->txt("edit_original_question"),
                '',
                $this->ctrl->getLinkTargetByClass('ilObjQuestionPoolGUI', "editQuestionForTest"),
                '',
                '',
                '_blank'
            );
            $this->ctrl->setParameterByClass('ilObjQuestionPoolGUI', 'ref_id', '');
            $this->ctrl->setParameterByClass('ilObjQuestionPoolGUI', 'q_id', '');

            $action->addItem(
                $this->plugin->txt("to_original_question_pool"),
                '',
                'goto.php?target=qpl_' . $row["pool_ref_id"] . '&client_id=' . CLIENT_ID,
                '',
                '',
                '_blank'
            );

            $row["actions"] = $action->getHTML();
        }

        return parent::formatCellValue($column, $row);
    }

    /**
     * @param array $data
     */
    protected function prepareData(array &$data)
    {
        $t0 = microtime(true);
        foreach ($data as $key => $item) {
            if (!is_array($item)) {
                $data[$key] = $item->toArray();
            }

            $tag = $data[$key]['type_tag'];
            $is_plugin = $data[$key]['plugin'];

            if (
                isset(self::$checked_qst_types[$tag]) &&
                self::$checked_qst_types[$tag] === false
            ) {
                unset($data[$key]);
                continue;
            } else {
                $is_supported = false;

                if (
                    $is_plugin &&
                    !$this->isPluginActive($tag)
                ) {
                    self::$checked_qst_types[$tag] = $is_supported;
                    unset($data[$key]);
                    continue;
                }

                assQuestion::_includeClass($tag);
                $reflection = new ReflectionClass($tag);
                if ($reflection->implementsInterface('iQuestionCondition')) {
                    $is_supported = true;
                }

                if (!$is_supported) {
                    unset($data[$key]);
                }

                self::$checked_qst_types[$tag] = $is_supported;
            }
        }
    }

    /**
     * Checks wheather or not a question plugin with a given name is active
     *
     * @param string $a_pname The plugin name
     * @access public
     */
    private function isPluginActive($a_pname)
    {
        global $DIC;
        $ilPluginAdmin = $DIC['ilPluginAdmin'];

        if ($ilPluginAdmin->isActive(IL_COMP_MODULE, "TestQuestionPool", "qst", $a_pname)) {
            return true;
        } else {
            return false;
        }
    }
}
