<?php

use PHPUnit\Framework\TestCase;
use PHPUnit\Framework\MockObject\MockObject;
use ILIAS\DI\Container;

/**
 * Class ilPluginDispatcherTest
 *
 * Date: 24.09.13
 * Time: 09:53
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilPluginDispatcherTest extends TestCase
{

    /**
     * @var ilPluginDispatcher
     */
    protected $dispatcher;

    /**
     * @var MockObject
     */
    protected $baseController_mock;

    public function setUp() : void
    {
        // Include original Object for mock, because an Error accures, if it is included in subfiles in a test
        require_once "./Services/UIComponent/Tabs/classes/class.ilTabsGUI.php";

        global $DIC;
        $DIC = new Container();
        $ilTabs_mock = $this->getMockBuilder(ilTabsGUI::class)->disableOriginalConstructor()->getMock();
        $GLOBALS["ilTabs"] = $ilTabs_mock;
        unset($DIC["ilTabs"]);
        $DIC["ilTabs"] = $GLOBALS["ilTabs"];

        $ilTabs_mock
            ->expects($this->any())
            ->method("activateTab");

        $this->baseController_mock = $this->getMockBuilder("ilObjQuestionSetPoolGUI")->disableOriginalConstructor()->getMock();
        $this->baseController_mock->plugin = new ilQuestionSetPoolPluginMock();



        $this->dispatcher = ilPluginDispatcher::getInstance($this->baseController_mock);
    }

    public function testGetInstance()
    {
        $dispatcher2 = ilPluginDispatcher::getInstance($this->baseController_mock);

        $this->assertNotNull($this->dispatcher);
        $this->assertEquals($this->dispatcher, $dispatcher2);
    }

    public function testGetController()
    {
        $getController = $this->getMethod("getController");

        $controller = $getController->invokeArgs($this->dispatcher, array("testController.testMethod"));

        $this->assertEquals("testController", $controller);
    }

    public function testGetCommand()
    {
        $getCommand = $this->getMethod("getCommand");

        $command = $getCommand->invokeArgs($this->dispatcher, array("testController.testMethod"));

        $this->assertEquals("testMethod", $command);
    }

    public function testGetCommandWithoutPassedController()
    {
        $getCommand = $this->getMethod("getCommand");

        $command = $getCommand->invokeArgs($this->dispatcher, array("testMethod"));

        $this->assertEquals("testMethod", $command);
    }

    public function testGetControllerWithoutPassedController()
    {
        $getController = $this->getMethod("getController");

        $controller = $getController->invokeArgs($this->dispatcher, array("testMethod"));

        $this->assertEquals($this->baseController_mock, $controller);
    }

    public function testInstantiateController()
    {
        $getController = $this->getMethod("getController");
        $instantiateController = $this->getMethod("instantiateController");

        $controllerName = $getController->invokeArgs($this->dispatcher, array("ilPluginSubControllerMock.testMethod"));
        $controller = $instantiateController->invokeArgs($this->dispatcher, array($controllerName));

        $this->assertInstanceOf("ilPluginSubControllerMock", $controller);
    }

    public function testGetControllerPath()
    {
        $getControllerPath = $this->getMethod("getControllerPath");

        $actual = $getControllerPath->invokeArgs($this->dispatcher, array());

        $this->assertStringEndsWith("/classes/controllers/", $actual);
    }

    public function testDispatch()
    {
        $this->expectOutputString("It work's");
        $this->dispatcher->dispatch("ilPluginSubControllerMock.testMethod");
    }

    /**
     * Get an ReflectionMethod to test private or protected functions
     *
     * @param string $method
     *
     * @return ReflectionMethod
     */
    private function getMethod($method)
    {
        $class = new ReflectionClass("ilPluginDispatcher");
        $method = $class->getMethod($method);
        $method->setAccessible(true);

        return $method;
    }
}
